@@ -11,8 +11,8 @@ Copyright (c) 2001-2003, Prabhu Ramachandran.
 """
 
 __author__ = "Prabhu Ramachandran <prabhu_r@users.sf.net>"
-__version__ = "$Revision: 1.11 $"
-__date__ = "$Date: 2003/10/22 20:21:48 $"
+__version__ = "$Revision: 1.13 $"
+__date__ = "$Date: 2004/07/14 21:10:24 $"
 __credits__ = """Many thanks to Jose Paulo <moitinho@civil.ist.utl.pt> 
 for adding tensor data support."""
 
@@ -20,6 +20,8 @@ for adding tensor data support."""
 import Tkinter, vtkpython, math, tkColorChooser
 import Common, vtkPipeline.vtkPipeline 
 import vtkPipeline.vtkMethodParser
+import re
+import glob
 
 debug = Common.debug
 
@@ -69,12 +71,12 @@ class VizObject (BaseObject):
     def _auto_sweep_init (self):
         """ Initialize variables for the autosweep GUI controls."""
         debug ("In VizObject::_auto_sweep_init ()")
-	self.sweep_var = Tkinter.IntVar ()
-	self.sweep_var.set (0)
-	self.sweep_step = Tkinter.IntVar ()
-	self.sweep_step.set (10)
-	self.sweep_delay = Tkinter.DoubleVar ()
-	self.sweep_delay.set (1.0)
+        self.sweep_var = Tkinter.IntVar ()
+        self.sweep_var.set (0)
+        self.sweep_step = Tkinter.IntVar ()
+        self.sweep_step.set (10)
+        self.sweep_delay = Tkinter.DoubleVar ()
+        self.sweep_delay.set (1.0)
 
     def _lift (self):
         """Lifts an already created configuration window to the
@@ -120,30 +122,32 @@ class VizObject (BaseObject):
                                   offvalue=0, command=self.show_pipeline)
         cb.pack(side='top', fill='both', expand=1)
 
-    def make_auto_sweep_gui (self):
+    def make_auto_sweep_gui (self, master=None):
         """ Create the GUI controls for the auto-sweep animation."""
         debug ("In VizObject::make_auto_sweep_gui ()")
-	frame = Tkinter.Frame (self.root, relief='ridge', bd=2)
-	frame.pack (side='top', fill='both', expand=1)
+        if master is None:
+            master = self.root
+        frame = Tkinter.Frame (master, relief='ridge', bd=2)
+        frame.pack (side='top', fill='both', expand=1)
 
-	cb = Tkinter.Checkbutton (frame, text='Auto Sweep',
+        cb = Tkinter.Checkbutton (frame, text='Auto Sweep',
                                   variable=self.sweep_var,
                                   command=self.do_sweep)
-	cb.grid (row=0, column=0, columnspan=2, sticky='w')
+        cb.grid (row=0, column=0, columnspan=2, sticky='w')
 
-	lab = Tkinter.Label (frame, text="Auto Sweep Step:")
-	lab.grid (row=1, column=0, sticky='w')
-	entry = Tkinter.Entry (frame, width=5, relief='sunken', 
+        lab = Tkinter.Label (frame, text="Auto Sweep Step:")
+        lab.grid (row=1, column=0, sticky='w')
+        entry = Tkinter.Entry (frame, width=5, relief='sunken', 
                                textvariable=self.sweep_step)
-	entry.grid (row=1, column=1, sticky='we')
-	entry.bind ("<Return>", self.sweep_step_validate)
+        entry.grid (row=1, column=1, sticky='we')
+        entry.bind ("<Return>", self.sweep_step_validate)
 
-	lab = Tkinter.Label (frame, text="Auto Sweep Delay:")
-	lab.grid (row=2, column=0, sticky='w')
-	entry = Tkinter.Entry (frame, width=5, relief='sunken', 
+        lab = Tkinter.Label (frame, text="Auto Sweep Delay:")
+        lab.grid (row=2, column=0, sticky='w')
+        entry = Tkinter.Entry (frame, width=5, relief='sunken', 
                                textvariable=self.sweep_delay)
-	entry.grid (row=2, column=1, sticky='we')
-	entry.bind ("<Return>", self.sweep_delay_validate)
+        entry.grid (row=2, column=1, sticky='we')
+        entry.bind ("<Return>", self.sweep_delay_validate)
         
     def make_close_button (self):
         """ Create a close button for the GUI control."""
@@ -182,8 +186,8 @@ class VizObject (BaseObject):
     def sweep_delay_validate(self, event=None):
         """Validate the sweep delay set by the user. """
         debug ("In VizObject::sweep_delay_validate ()")
-	if self.sweep_delay.get() <= 0:
-	   self.sweep_delay.set(1.0)
+        if self.sweep_delay.get() <= 0:
+           self.sweep_delay.set(1.0)
 
     def close_gui (self, event=None):
         """Called when the 'close' button is clicked."""
@@ -209,6 +213,38 @@ class Source (VizObject):
         """Output of the vtkSource object wrapped."""
         debug ("In Source::GetOutput ()")
         debug ("Source::GetOutput not implemented.")
+
+
+def get_file_list (file_name):
+    
+    """ Given a file name, this function treats the file as a part of
+    a series of files based on the index of the file and tries to
+    determine the list of files in the series.  The file name of a
+    file in a time series must be of the form 'some_name[0-9]*.ext'.
+    That is the integers at the end of the file determine what part of
+    the time series the file belongs to.  The files are then sorted as
+    per this index."""
+    
+    debug ("In get_timestep_list ()")
+    
+    head = re.sub("[0-9]+[^0-9]*$", "", file_name)
+    tail = re.sub("^.*[0-9]+", "", file_name)
+    pattern = head+"[0-9]*"+tail
+    
+    files = glob.glob (pattern)
+
+    def file_sort(x, y):
+        x1 = float(re.sub(tail + '$', '', re.sub('^'+head, '', x)))
+        y1 = float(re.sub(tail + '$', '', re.sub('^'+head, '', y)))
+        if x1 > y1:
+            return 1
+        elif y1 > x1:
+            return -1
+        else:
+            return 0
+
+    files.sort(file_sort)
+    return files
     
 
 class DataSource (Source):
@@ -224,19 +260,50 @@ class DataSource (Source):
         self.vector_data_name = ""        
         self.tensor_data_name = ""
         self.ref_lst = []
+        self.file_list = [] # used if the file is part of a time series
+        self.timestep = 0
         self.grid_type = ""
         self.root = None
     
     def initialize (self, file_name): 
         """Initialize the object given a valid file name."""
         debug ("In DataSource::initialize ()")
-        debug ("DataSource::initialize not implemented.")
+        self.file_list = get_file_list(file_name)
+        try:
+            self.timestep = self.file_list.index(file_name)
+        except ValueError:
+            pass
+        if len(self.file_list) > 1:
+            self._auto_sweep_init()
+            self.sweep_step.set(1)
+
+    def set_file_name(self, file_name):
+        """Abstracts the call to self.reader.Set*FileName(). Useful
+        when using this class as a base for other classes.  Override
+        this if needed."""
+        debug ("In DataSource::set_file_name ()")
+        debug ("DataSource::set_file_name () is not implemented")        
 
     def get_file_name (self):
         """Get the data file name. """
         debug ("In DataSource::get_file_name ()")
         return self.file_name
 
+    def get_file_list (self):
+        """Get the list of data files. """
+        debug ("In DataSource::get_file_list ()")
+        return self.file_list
+
+    def get_timestep (self):
+        return self.timestep
+
+    def set_timestep (self, timestep):
+        """Used to change the time level of the file if it is part of
+        a time series.  The time step must be an integer."""
+        debug ("In DataSource::set_timestep ()")
+        self.set_file_name(self.file_list[timestep])
+        self.timestep = timestep
+
     def add_reference (self, ref):
         """ Add a reference object that is notified each time anything
         is changed in the DataSource object.  This is used for all
@@ -272,30 +339,75 @@ class DataSource (Source):
         pass
 
     def GetOutput (self): 
-	"""Get the Data reader's output. """
+        """Get the Data reader's output. """
         debug ("In DataSource::GetOutput ()")
         pass
 
     def get_grid_type (self): 
-	"""Get the type of grid used in the data file."""
+        """Get the type of grid used in the data file."""
         debug ("In DataSource::get_grid_type ()")
-	return self.grid_type
+        return self.grid_type
 
     def get_scalar_data_name (self): 
-	"""Get the name of the scalar data attribute being used."""
+        """Get the name of the scalar data attribute being used."""
         debug ("In DataSource::get_scalar_data_name ()")
-	return self.scalar_data_name
+        return self.scalar_data_name
 
     def get_vector_data_name (self): 
-	"""Get the name of the vector data attribute being used."""
+        """Get the name of the vector data attribute being used."""
         debug ("In DataSource::get_vector_data_name ()")
-	return self.vector_data_name
+        return self.vector_data_name
 
     def get_tensor_data_name (self):
         """Get the name of the tensor data attribute being used."""
         debug ("In DataSource::get_tensor_data_name ()")
         return self.tensor_data_name
 
+    def make_timestep_gui (self, master):
+        debug ("In DataSource::make_timestep_gui()")
+        nf = len(self.file_list)
+        if nf < 2:
+            return
+        
+        sl = Tkinter.Scale (master, label="Set Time Step", 
+                            from_=0, to=nf-1,
+                            length="5c", orient='horizontal', 
+                            resolution=1)
+        sl.set (self.timestep)
+        sl.pack (side='top', fill='both', expand=1)
+        sl.bind ("<ButtonRelease>", self.set_timestep_gui)
+        self.timestep_var = sl
+        
+        self.make_auto_sweep_gui(master)
+
+    def do_sweep (self, event=None):
+        """Called when the user requests auto-sweeping."""
+        debug ("In DataSource::do_sweep ()")
+        if self.sweep_var.get ():
+            val = int (1000*self.sweep_delay.get ())
+            self.root.after (val, self.update_sweep)
+            
+    def update_sweep (self):
+        debug ("In DataSource::update_sweep ()")
+        if self.sweep_var.get ():
+            val = int (1000*self.sweep_delay.get ())
+            timestep = (self.timestep + self.sweep_step.get())%len(self.file_list)
+            self.timestep_var.set (timestep)
+            self.set_timestep_gui ()
+            self.root.after (val, self.update_sweep)
+    
+    def set_timestep_gui (self, event=None): 
+        debug ("In DataSource::set_timestep_gui ()")
+        timestep = self.timestep_var.get()
+        if timestep == self.timestep:
+            return
+        Common.state.busy ()
+        self.set_timestep (timestep)
+        self.reread_file ()
+        self.file_name_label.config(text=self.file_name)
+        Common.state.idle ()
+
+
 class Filter (Source):
 
     """ The basic Filter class from which all are filters
@@ -336,23 +448,23 @@ class Filter (Source):
         return self.fil.GetOutput ()    
 
     def get_scalar_data_name (self):
-	"""Get the name of the scalar data attribute being used.
-	Defaults to returning the previous filters scalar
-	data_name."""
+        """Get the name of the scalar data attribute being used.
+        Defaults to returning the previous filters scalar
+        data_name."""
         debug ("In Filter::get_scalar_data_name ()")
         return self.prev_fil.get_scalar_data_name ()
         
     def get_vector_data_name (self):
-	"""Get the name of the vector data attribute being used.
-	Defaults to returning the previous filters vector
-	data_name."""
+        """Get the name of the vector data attribute being used.
+        Defaults to returning the previous filters vector
+        data_name."""
         debug ("In Filter::get_vector_data_name ()")
         return self.prev_fil.get_vector_data_name ()
 
     def get_tensor_data_name (self):
         """Get the name of the tensor data attribute being used.
         Defaults to returning the previous filters tensor
-	data_name."""
+        data_name."""
         debug ("In Filter::get_tensor_data_name ()")
         return self.prev_fil.get_tensor_data_name ()
 
@@ -563,15 +675,15 @@ class Module (VizObject):
     def change_color (self, event=None): 
         """ Called when the user changes the color of the actor."""
         debug ("In Module::change_color ()")
-	clr = self.actor.GetProperty ().GetColor ()
-	init_clr = "#%02x%02x%02x"%(clr[0]*255, clr[1]*255, clr[2]*255)
-	color = tkColorChooser.askcolor (title="Change object color", 
-					 initialcolor=init_clr)
-	if color[1] is not None:
+        clr = self.actor.GetProperty ().GetColor ()
+        init_clr = "#%02x%02x%02x"%(clr[0]*255, clr[1]*255, clr[2]*255)
+        color = tkColorChooser.askcolor (title="Change object color", 
+                                         initialcolor=init_clr)
+        if color[1] is not None:
             Common.state.busy ()
-	    clr = Common.tk_2_vtk_color (color[0])
-	    self.actor.GetProperty ().SetColor (*clr)
-	    self.renwin.Render ()        
+            clr = Common.tk_2_vtk_color (color[0])
+            self.actor.GetProperty ().SetColor (*clr)
+            self.renwin.Render ()        
             Common.state.idle ()
             if self.scalar_on_var.get () != 0:
                 msg = "Warning: This setting will have effect only if "\
@@ -634,8 +746,8 @@ class CutPlaneModule (Module):
         Module.__init__ (self, mod_m)
         # this should be a vtkPlane
         self.plane = vtkpython.vtkPlane ()
-	self.step_var = Tkinter.DoubleVar ()
-	self.n_step_var = Tkinter.IntVar ()
+        self.step_var = Tkinter.DoubleVar ()
+        self.n_step_var = Tkinter.IntVar ()
         self.resoln_var = Tkinter.DoubleVar ()
         self.resoln_var.set (1.0)
         self.slider = []
@@ -698,55 +810,55 @@ class CutPlaneModule (Module):
         sl.bind ("<ButtonRelease>", self.change_slider)
         self.slider.append (sl)
 
-	f = Tkinter.Frame (frame)
-	f.pack (side='top')
+        f = Tkinter.Frame (frame)
+        f.pack (side='top')
 
         rw = 0
-	labl = Tkinter.Label (f, text="Angle resolution:")
-	labl.grid (row=rw, column=0, sticky='w')
-	self.step_entr = Tkinter.Entry (f, width=5, relief='sunken',
+        labl = Tkinter.Label (f, text="Angle resolution:")
+        labl.grid (row=rw, column=0, sticky='w')
+        self.step_entr = Tkinter.Entry (f, width=5, relief='sunken',
                                         textvariable=self.resoln_var)
-	self.step_entr.grid (row=rw, column=1, sticky='we')
-	self.step_entr.bind ("<Return>", self.change_resoln)
+        self.step_entr.grid (row=rw, column=1, sticky='we')
+        self.step_entr.bind ("<Return>", self.change_resoln)
         rw = rw + 1
 
-	labl = Tkinter.Label (f, text="Distance step:")
-	labl.grid (row=rw, column=0, sticky='w')
-	self.step_entr = Tkinter.Entry (f, width=5, relief='sunken',
+        labl = Tkinter.Label (f, text="Distance step:")
+        labl.grid (row=rw, column=0, sticky='w')
+        self.step_entr = Tkinter.Entry (f, width=5, relief='sunken',
                                         textvariable=self.step_var)
-	self.step_entr.grid (row=rw, column=1, sticky='we')
-	self.step_entr.bind ("<Return>", self.change_cut)
+        self.step_entr.grid (row=rw, column=1, sticky='we')
+        self.step_entr.bind ("<Return>", self.change_cut)
         rw = rw + 1
 
-	labl = Tkinter.Label (f, text="Step bounds:")
-	labl.grid (row=rw, column=0, sticky='w')
-	self.n_step_entr = Tkinter.Entry (f, width=5, relief='sunken',
+        labl = Tkinter.Label (f, text="Step bounds:")
+        labl.grid (row=rw, column=0, sticky='w')
+        self.n_step_entr = Tkinter.Entry (f, width=5, relief='sunken',
                                           textvariable=self.n_step_var)
-	self.n_step_entr.grid (row=rw, column=1, sticky='we')
-	self.n_step_entr.bind ("<Return>", self.change_cut)
+        self.n_step_entr.grid (row=rw, column=1, sticky='we')
+        self.n_step_entr.bind ("<Return>", self.change_cut)
     
     def change_cut (self, event=None):
         debug ("In CutPlaneModule::change_cut ()")
-	self.step_size = self.step_var.get ()
-	n_s = self.n_step_var.get ()
-	self.slider[2].config (from_=-n_s, to=n_s)
-	self.change_slider (event)
+        self.step_size = self.step_var.get ()
+        n_s = self.n_step_var.get ()
+        self.slider[2].config (from_=-n_s, to=n_s)
+        self.change_slider (event)
 
     def change_slider (self, event=None):
         debug ("In CutPlaneModule::change_slider ()")
         Common.state.busy ()
-	val = []
-	for i in range (3):
-	    val.append (self.slider[i].get ())
+        val = []
+        for i in range (3):
+            val.append (self.slider[i].get ())
 
         self.slider_pos = val[2]
-	self.config_extents (val)
-	self.renwin.Render ()
+        self.config_extents (val)
+        self.renwin.Render ()
         Common.state.idle ()    
 
     def change_resoln (self, event=None):
         debug ("In CutPlaneModule::change_resoln ()")
         val = self.resoln_var.get ()
-	for i in range (0, 2):
-	    self.slider[i].config (resolution=val)        
+        for i in range (0, 2):
+            self.slider[i].config (resolution=val)        
     
