@@ -17,8 +17,8 @@ Copyright (c) 2001-2002, Prabhu Ramachandran.
 """
 
 __author__ = "Prabhu Ramachandran <prabhu_r@users.sf.net>"
-__version__ = "$Revision: 1.13 $"
-__date__ = "$Date: 2003/11/05 19:53:48 $"
+__version__ = "$Revision: 1.16 $"
+__date__ = "$Date: 2004/07/06 23:06:31 $"
         
 import os, sys, string, pickle
 import Tkinter, tkColorChooser, tkMessageBox, tkFileDialog
@@ -109,8 +109,13 @@ class AppConfig:
                         "stereo": 0,
                         "light_cfg": None,
                         "scalar_lut_cfg": None,
-                        "vector_lut_cfg": None}
+                        "vector_lut_cfg": None,
+                        "search_path":'',
+                        }
             self.__dict__['data'] = defaults
+            self.__dict__['upath'] = []
+            self.__dict__['upath_filters'] = []
+            self.__dict__['upath_modules'] = []            
         
             if os.path.isfile (self.config_file):
                 f = open (self.config_file, 'r')
@@ -124,6 +129,26 @@ class AppConfig:
                     # initial_dir == '' is invalid on the Mac.
                     self.initial_dir = os.getcwd()
 
+            self.update_search_paths()
+
+    def update_search_paths(self):
+
+        # Search also the user's search path as set in the Preferences
+        # This is like PYTHONPATH, a ':'-separated string.  ~, ~user
+        # and $VAR are all expanded.        
+        spath = self.search_path.split(':')
+        spath = map(os.path.expandvars, spath)
+        spath = map(os.path.expanduser, spath)
+        
+        self.__dict__['upath'] = filter(os.path.isdir, spath)
+        uf = []
+        um = []
+        for user_dir in self.upath:
+            uf.append(os.path.join(user_dir, 'Filters'))
+            um.append(os.path.join(user_dir, 'Modules'))
+        self.__dict__['upath_filters'] = uf
+        self.__dict__['upath_modules'] = um
+
     def __getattr__ (self, name):
         return self.__dict__['data'][name]
 
@@ -135,6 +160,7 @@ class AppConfig:
         f = open (self.config_file, 'w')
         f.write ("MayaVi Config file.\n")
         pickle.dump (self.data, f)
+        f.close()
 
 
 def tk_2_vtk_color (tk_col):
@@ -167,6 +193,9 @@ class EditConfig:
         self.save_lights.set(0)
         self.save_sc_lut.set(0)
         self.save_vec_lut.set(0)
+
+        self.search_path = Tkinter.StringVar ()
+        self.search_path.set(str(cfg.search_path))
         
         self.root = Tkinter.Toplevel (master)
         self.root.transient (self.root.master)
@@ -207,6 +236,13 @@ class EditConfig:
         entr.grid (row=rw, column=1, sticky='ew', pady=2)
         rw += 1
 
+        lab = Tkinter.Label (frame, text="Search path:", padx=1)
+        lab.grid (row=rw, column=0, sticky='w', pady=2)
+        entr = Tkinter.Entry (frame, width=20, relief='sunken', 
+                              textvariable=self.search_path)
+        entr.grid (row=rw, column=1, sticky='ew', pady=2)
+        rw += 1
+
         sl = Tkinter.Scale (frame,
                             label="Magnification factor for saving images",
                             from_=1, to=10, length="8c",
@@ -274,7 +310,7 @@ class EditConfig:
         if new_color[1] != None:
             col = tk_2_vtk_color (new_color[0])
             self.fg_color.set (col)
-
+        
     def get_dir (self, event=None):
         """Choose a directory from a file selected."""
         msg = "Please choose a file in the chosen directory.  "\
@@ -305,6 +341,8 @@ class EditConfig:
                 )
             return 0
         self.cfg.initial_dir = direc
+        self.cfg.search_path = self.search_path.get()
+        self.cfg.update_search_paths()
 
         # Stereo
         rw = self.app.get_render_window()
@@ -509,4 +547,40 @@ def debug (msg):
     log_win.write_log (msg)
 
 
+def mod_fil_import(kind, name, globals, locals, user_paths=0):
+    """
+    Special importer which tries alternate paths.  Used to load
+    Filters and Modules from user paths as well as MayaVi's internal
+    one.  If the argument user_paths is true then the modules/filters
+    are first searched for in the user specified paths obtained from
+    config.search_path.
+    """
+    # User paths should override system defaults, so users can customize
+    # their modules without system-wide write access
+    assert kind in ['Modules', 'Filters']
+    
+    extra_path = ''
+    if user_paths:
+        if kind == 'Modules':
+            extra_path = config.upath_modules
+        elif kind == 'Filters':
+            extra_path = config.upath_filters
+    try:
+        if extra_path:
+            try:
+                spath = sys.path
+                sys.path = extra_path + sys.path
+                mod_scr = __import__ (name,globals)
+            finally:
+                sys.path = spath
+        else:
+            # so that the normal import kicks in
+            raise ImportError
+    # If a module isn't in the user's path
+    except ImportError:
+        mod_scr = __import__ ("%s.%s"%(kind, name), globals,
+                              locals, [name])
+    return mod_scr
+
+
 del _app_cfg, _app_state, _log_data
