@@ -11,12 +11,12 @@ Bugs:
 This code is distributed under the conditions of the BSD license.  See
 LICENSE.txt for details.
 
-Copyright (c) 2001-2002, Prabhu Ramachandran.
+Copyright (c) 2001-2005, Prabhu Ramachandran.
 """
 
 __author__ = "Prabhu Ramachandran <prabhu_r@users.sf.net>"
-__version__ = "$Revision: 1.38 $"
-__date__ = "$Date: 2003/11/08 12:09:41 $"
+__version__ = "$Revision: 1.43 $"
+__date__ = "$Date: 2005/02/01 16:16:06 $"
 
 import os, traceback, glob, sys, string
 import Tkinter, tkFileDialog, tkMessageBox, tkColorChooser
@@ -41,8 +41,7 @@ def reload_all_modules ():
 
     """This function reloads all the currently loaded modules.  _Very_
     useful while debugging.  Please note that the configuration
-    settings from Common.config will remain unchanged.  You _might_
-    also see funny behaviour for already instantiated objects."""
+    settings from Common.config will remain unchanged.  You _might_    also see funny behaviour for already instantiated objects."""
 
     debug ("In reload_all_modules")
     msg = "Warning: This reloads all the currently loaded modules. "\
@@ -84,9 +83,19 @@ def exception ():
         type = value = tb = None # clean up
         
 
-def find_scripts (s_dir="Modules"):
+def find_scripts (s_dir="Modules", extra_dirs=None, no_base=0):
     """Finds all scripts in the directory specified in the
-    argument."""    
+    argument.
+
+    Keyword arguments:
+
+    extra_dirs -- List of directories to search for additional
+    modules/filters.  Defaults to None.
+
+    no_base -- If true, does not search for modules and filters inside
+    the base directory.  Defaults to false.
+    
+    """    
     debug ("In find_scripts ()")
     # this handles frozen installs made using the Python Installer,
     home, exe = os.path.split (sys.executable)
@@ -95,13 +104,24 @@ def find_scripts (s_dir="Modules"):
         base_dir = os.path.abspath (os.path.dirname (Common.__file__))
     else:
         base_dir = os.path.abspath (home)
-        
-    lst = glob.glob ("%s/%s/*.py"%(base_dir, s_dir))
+
+    if not no_base:
+        lst = glob.glob ("%s/%s/*.py"%(base_dir, s_dir))
+    else:
+        lst = []
+
+    if extra_dirs is not None:
+        for dd in extra_dirs:
+            lst.extend(glob.glob ("%s/%s/*.py"%(dd, s_dir)))
+
     ret = []
     for name in lst:
         i = os.path.basename (name)
         ret.append (os.path.splitext (i)[0])
-    ret.remove ("__init__")
+    try:
+        ret.remove ("__init__")
+    except ValueError:
+        pass
     ret.sort ()
     return ret
 
@@ -144,16 +164,16 @@ def check_file (file_name):
 class MayaVi:
     
     """ This is the main MayaVi class.  This does all the dirty stuff.
-    The GUI class merely wraps around this"""
+    The GUI class merely wraps around this."""
 
     n_app = 0
     def __init__ (self, gui): 
         debug ("In MayaVi::__init__ ()")
-	self.gui = gui
+        self.gui = gui
         MayaVi.n_app = MayaVi.n_app + 1
         n = MayaVi.n_app
 
-	self.data_viz_mgr = {}
+        self.data_viz_mgr = {}
         self.dvm_name = []
         self.cur_dvm_name = ""
         self.n_dvm = 0
@@ -422,7 +442,28 @@ class MayaVi:
         else:
             rw.Render ()        
 
-    def load_module (self, mod_name, config=1, *args, **kw_args):
+    def load_module (self, mod_name, config=1, *args, **kw_args):        
+        """ Loads a MayaVi module given the following arguments.  
+
+        Keyword arguments:
+
+        mod_name -- Name of the module (a string).  If the module name
+        has a 'User.' prepended to it then the module is first
+        searched for in the user specified search paths and then in
+        the default system directories.  The search path can be
+        specified like PYTHONPATH, and is a ':'-separated string.  ~,
+        ~user and $VAR are all expanded.  The path can be specified in
+        the Preferences dialog.
+        
+        config -- If true (default) shows a configuration GUI for the
+        module as soon as it is loaded.
+
+        args -- Positional arguments passed on to the module's
+        constructor.        
+
+        kw_args -- Keyword arguments passed on to the module's
+        constructor.
+        """        
         debug ("In MayaVi::load_module ()")
         if not self.dvm_name:
             msg = "You need to have some data opened to be able to "\
@@ -442,9 +483,17 @@ class MayaVi:
             return
         try:
             Common.state.busy ()
-            mod_scr = __import__ ("Modules.%s"%mod_name, globals (),
-                                  locals (), [mod_name])
+            _imp = Common.mod_fil_import
+            if mod_name[:5] == 'User.':
+                mod_scr = _imp('Modules', mod_name[5:], globals(),
+                               locals(), 1)
+                mod_name = mod_name[5:]
+            else:
+                mod_scr = _imp('Modules', mod_name, globals(),
+                               locals(), 0)
+                
             m = eval ("mod_scr.%s"%mod_name)(mm, *args, **kw_args)
+
             mm.add_module_gui (m)
             if config:
                 m.configure (self.gui.root)
@@ -456,6 +505,27 @@ class MayaVi:
             Common.state.force_idle ()
 
     def load_filter (self, fil_name, config=1, *args, **kw_args):
+        """ Loads a MayaVi filter given the following arguments.
+
+        Keyword arguments:
+
+        fil_name -- Name of the filter (a string).  If the filter name
+        has a 'User.' prepended to it then the filter is first
+        searched for in the user specified search paths and then in
+        the default system directories.  The search path can be
+        specified like PYTHONPATH, and is a ':'-separated string.  ~,
+        ~user and $VAR are all expanded.  The path can be specified in
+        the Preferences dialog.
+        
+        config -- If true (default) shows a configuration GUI for the
+        module as soon as it is loaded.
+        
+        args -- Positional arguments passed on to the filter's
+        constructor.
+        
+        kw_args -- Keyword arguments passed on to the filter's
+        constructor.
+        """        
         debug ("In MayaVi::load_filter ()")
         if not self.dvm_name:
             msg = "You need to have some data opened to be able to "\
@@ -475,15 +545,21 @@ class MayaVi:
             return
         try:
             Common.state.busy ()
-            if fil_name[:12] == 'UserDefined:':
+            _imp = Common.mod_fil_import
+            if fil_name[:5] == 'User.':
+                fil_name = fil_name[5:]
+                fil_scr = _imp("Filters", fil_name, globals(),
+                               locals (), 1)
+                f = eval ("fil_scr.%s"%fil_name)(mm, *args, **kw_args)
+            elif fil_name[:12] == 'UserDefined:':
                 name = fil_name[:11]
-                fil_scr = __import__ ("Filters.%s"%name, globals (),
-                                      locals (), [name])
+                fil_scr = _imp("Filters", name, globals(),
+                               locals (), 0)
                 vtk_fil = fil_name[12:]
                 f = eval ("fil_scr.%s (mm, vtk_fil, *args, **kw_args)"%fil_name[:11])
             else:
-                fil_scr = __import__ ("Filters.%s"%fil_name, globals (),
-                                      locals (), [fil_name])
+                fil_scr = _imp("Filters", fil_name, globals(),
+                               locals (), 0)
                 f = eval ("fil_scr.%s"%fil_name)(mm, *args, **kw_args)
             
             mm.add_filter_gui (f)
@@ -777,7 +853,7 @@ class DocHelper:
               "Version: %s\n\n"\
               "License: BSD\n\n"\
               "Home page: http://mayavi.sourceforge.net\n\n"\
-              "(c) Prabhu Ramachandran, 2000-2002\n"%(version,)
+              "(c) Prabhu Ramachandran, 2000-2005\n"%(version,)
         return tkMessageBox.showinfo ("About MayaVi", msg)
 
     def show_home_page (self):
@@ -809,18 +885,30 @@ class MayaViTkGUI:
     """ This is the main application and creates the GUI and controls
     everything."""
 
-    def __init__ (self, master): 
+    def __init__ (self, master, geometry=None):
+        """
+        Constructor for the main MayaVi GUI window.
+        
+        Keyword Arguments:
+
+        master -- Toplevel parentwindow.        
+        geometry -- The geometry of the main window in standard X
+        fashion (WxH+X+Y).
+
+        """
         debug ("In MayaViTkGUI::__init__ ()")
-	self.master = master
-	self.root = Tkinter.Toplevel (master)        
+        self.master = master
+        self.root = Tkinter.Toplevel (master)        
         self.mayavi = MayaVi (self)
         self.doc_helper = DocHelper ()
         
         n = MayaVi.n_app
-	self.root.title ("MayaVi Data Visualizer %d"%n)
+        self.root.title ("MayaVi Data Visualizer %d"%n)
         self.root.protocol ("WM_DELETE_WINDOW", self.quit)
-	self.root.minsize (650, 600)
-	#self.root.geometry ("600x600+0+0")
+        if geometry:
+            self.root.geometry(geometry)
+        self.root.minsize (650, 600)
+        #self.root.geometry ("600x600+0+0")
         master_f = Tkinter.Frame (self.root, relief='sunken', bd=2)
         master_f.pack (side='top', fill='both', expand=1)
 
@@ -853,7 +941,7 @@ class MayaViTkGUI:
         self.full_scr_var = Tkinter.IntVar ()
         self.full_scr_var.set (1)
 
-	self.make_menus ()
+        self.make_menus ()
         self.make_data_list ()
 
     def __del__ (self):
@@ -871,11 +959,11 @@ class MayaViTkGUI:
     def make_menus (self): 
         debug ("In MayaViTkGUI::make_menus ()")
         self.menu = Tkinter.Menu (self.root, tearoff=0)
-	self.root.config (menu=self.menu)
+        self.root.config (menu=self.menu)
 
-	self.file_menu = Tkinter.Menu (self.menu, name='file', tearoff=0)
-	self.menu.add_cascade (label="File", menu=self.file_menu, 
-			       underline=0)
+        self.file_menu = Tkinter.Menu (self.menu, name='file', tearoff=0)
+        self.menu.add_cascade (label="File", menu=self.file_menu, 
+                               underline=0)
         self.viz_menu = Tkinter.Menu (self.menu, name='visualize',
                                       tearoff=0)
         self.menu.add_cascade (label="Visualize", menu=self.viz_menu,
@@ -888,7 +976,7 @@ class MayaViTkGUI:
         self.menu.add_cascade (label="Help", menu=self.help_menu,
                                underline=0)
 
-	## File menus
+            ## File menus
         self.file_menu.add_command (label="New Window", underline=0, 
                                     command=self.new_window)        
 
@@ -921,12 +1009,12 @@ class MayaViTkGUI:
                                     command=self.quit)
         
         ## Open menu
-	self.open_menu.add_command (label="VTK file", underline=0,
-				    command=self.open_vtk)
-	self.open_menu.add_command (label="VTK XML file", underline=4,
-				    command=self.open_vtk_xml)
-	self.open_menu.add_command (label="EnSight case file", underline=0,
-				    command=self.open_ensight)
+        self.open_menu.add_command (label="VTK file", underline=0,
+                                    command=self.open_vtk)
+        self.open_menu.add_command (label="VTK XML file", underline=4,
+                                    command=self.open_vtk_xml)
+        self.open_menu.add_command (label="EnSight case file", underline=0,
+                                    command=self.open_ensight)
         self.pl3d_menu = Tkinter.Menu (self.open_menu, tearoff=0)
         self.open_menu.add_cascade (label="PLOT3D file", underline=1,
                                     menu=self.pl3d_menu)
@@ -1028,6 +1116,7 @@ class MayaViTkGUI:
         self.filter_menu = Tkinter.Menu (self.viz_menu, tearoff=0)
         self.viz_menu.add_cascade (label="Filters", underline=0,
                                    menu=self.filter_menu)
+
         self.make_module_menu ()
         self.make_filter_menu ()
         self.viz_menu.add_command (label="Pipeline browser", underline=0,
@@ -1074,20 +1163,38 @@ class MayaViTkGUI:
     def make_module_menu (self):
         debug ("In MayaViTkGUI::make_module_menu ()")
         mod_list = find_scripts ("Modules")
+        mod_list.insert(0, 'User')
         hk_list = find_hot_key (mod_list)
-        for i in range (len (mod_list)):
+        for i in range (1, len (mod_list)):
             mod = mod_list[i]
             hk = hk_list[i]
             self.module_menu.add_radiobutton (label=mod, value=mod,
                                               variable=self.module_var,
                                               indicatoron=0, underline=hk,
                                               command=self.load_module)
+        
+        self.user_module_menu = Tkinter.Menu (self.viz_menu, tearoff=0)
+        self.module_menu.add_cascade (label="User", underline=hk_list[0],
+                                      menu=self.user_module_menu)
+        
+        mod_list = find_scripts ("Modules", Common.config.upath, no_base=1)
+        hk_list = find_hot_key (mod_list)        
+        for i in range (len (mod_list)):
+            mod = mod_list[i]
+            hk = hk_list[i]
+            self.user_module_menu.add_radiobutton (label=mod,
+                                                   value='User.'+mod,
+                                                   variable=self.module_var,
+                                                   indicatoron=0,
+                                                   underline=hk,
+                                                   command=self.load_module)
 
     def make_filter_menu (self):
         debug ("In MayaViTkGUI::make_filter_menu ()")
         fil_list = find_scripts ("Filters")
+        fil_list.insert(0, 'User')
         hk_list = find_hot_key (fil_list)
-        for i in range (len (fil_list)):
+        for i in range (1, len (fil_list)):
             fil = fil_list[i]
             hk = hk_list[i]
             self.filter_menu.add_radiobutton (label=fil, value=fil,
@@ -1095,18 +1202,34 @@ class MayaViTkGUI:
                                               indicatoron=0, underline=hk,
                                               command=self.load_filter)
 
+        self.user_filter_menu = Tkinter.Menu (self.viz_menu, tearoff=0)
+        self.filter_menu.add_cascade (label="User", underline=hk_list[0],
+                                      menu=self.user_filter_menu)
+        
+        fil_list = find_scripts ("Filters", Common.config.upath, no_base=1)
+        hk_list = find_hot_key (fil_list)        
+        for i in range (len (fil_list)):
+            fil = fil_list[i]
+            hk = hk_list[i]
+            self.user_filter_menu.add_radiobutton (label=fil,
+                                                   value='User.'+fil,
+                                                   variable=self.filter_var,
+                                                   indicatoron=0,
+                                                   underline=hk,
+                                                   command=self.load_filter)
+
     def make_data_list (self): 
         debug ("In MayaViTkGUI::make_data_list ()")
         frame = Tkinter.Frame (self.ctrl_frame)
         frame.pack (side='top', fill='y', expand=0)
         scr = Tkinter.Scrollbar (frame, orient='vertical')
-	self.dvm_lst = Tkinter.Listbox (frame, yscrollcommand=scr.set, 
+        self.dvm_lst = Tkinter.Listbox (frame, yscrollcommand=scr.set, 
                                         selectmode='single', height=3,
                                         exportselection=0)
-	scr.config (command=self.dvm_lst.yview)
+        scr.config (command=self.dvm_lst.yview)
         self.dvm_lst.grid (row=0, column=0, sticky='ewns')
         scr.grid (row=0, column=1, sticky='ns')
-	self.dvm_lst.bind ("<Double-Button-1>", self.show_dvm)
+        self.dvm_lst.bind ("<Double-Button-1>", self.show_dvm)
 
         but_f = Tkinter.Frame (self.ctrl_frame)
         but_f.pack (side='top', fill='y', expand=0)
@@ -1349,8 +1472,7 @@ class MayaViTkGUI:
         debug ("In MayaViTkGUI::load_module ()")
         if not mod_name:
             mod_name = self.module_var.get ()
-        return self.mayavi.load_module(mod_name, config=config, *args,
-                                       **kw_args)
+        return self.mayavi.load_module(mod_name, config, *args, **kw_args)
 
     def load_filter (self, fil_name="", config=1, *args, **kw_args):
         """Loads a Filter.
@@ -1375,8 +1497,7 @@ class MayaViTkGUI:
         debug ("In MayaViTkGUI::load_filter ()")
         if not fil_name:
             fil_name = self.filter_var.get ()
-        return self.mayavi.load_filter(fil_name, config=config, *args,
-                                       **kw_args)
+        return self.mayavi.load_filter(fil_name, config, *args, **kw_args)
 
     def load_current_mm (self, file_name=""):
         """Load a ModuleManager from a file. This function loads the
@@ -1689,16 +1810,22 @@ class MayaViTkGUI:
         self.root.after (*l)
 
 
-def mayavi ():    
+def mayavi (geometry=None):
     """ Returns a MayaViTkGUI Window.  This is to be used when using
-    MayaVi from the Python interpreter."""    
+    MayaVi from the Python interpreter.
+
+    Keyword Arguments:
+
+    geometry -- The geometry of the main window in standard X
+    fashion (WxH+X+Y).
+    """    
     r = Tkinter._default_root
     if not r:
         r = Tkinter.Tk ()
         r.withdraw ()
     t = Tkinter.Toplevel (r)
     t.withdraw ()
-    app = MayaViTkGUI (t)
+    app = MayaViTkGUI (t, geometry)
     return app
 
 
