@@ -12,8 +12,8 @@ Copyright (c) 2001-2003, Prabhu Ramachandran.
 """
 
 __author__ = "Prabhu Ramachandran <prabhu_r@users.sf.net>"
-__version__ = "$Revision: 1.23 $"
-__date__ = "$Date: 2003/11/06 07:15:42 $"
+__version__ = "$Revision: 1.25 $"
+__date__ = "$Date: 2004/07/06 04:29:30 $"
 
 import Tkinter, tkFileDialog, tkMessageBox, tkSimpleDialog
 import vtkpython, vtkRenderWidget
@@ -78,6 +78,7 @@ class RenderWindow (Base.Objects.VizObject):
         self.frame = Tkinter.Frame (master)
         self.frame.pack (side='top', fill='both', expand=1)
         tkw = None
+        self._do_render = 1
         stereo = Common.config.stereo
         if stereo:
             tkw = vtkRenderWidget.vtkTkRenderWidget (self.frame, width=600,
@@ -86,43 +87,43 @@ class RenderWindow (Base.Objects.VizObject):
             tkw = vtkRenderWidget.vtkTkRenderWidget (self.frame, width=600,
                                                      height=505)
         self.tkwidget = tkw
-	self.tkwidget.pack (expand='true',fill='both')
-	#self.tkwidget.bind ("<KeyPress-q>", self.quit)
+        self.tkwidget.pack (expand='true',fill='both')
+        #self.tkwidget.bind ("<KeyPress-q>", self.quit)
         # disabling surface and wireframe toggling.
         self.tkwidget.unbind ("<KeyPress-s>")
         self.tkwidget.unbind ("<KeyPress-w>")
-	self.ren = vtkpython.vtkRenderer ()
+        self.ren = vtkpython.vtkRenderer ()
         self.ren.TwoSidedLightingOn ()
-	self.renwin = self.tkwidget.GetRenderWindow ()
-	self.renwin.AddRenderer (self.ren)
+        self.renwin = self.tkwidget.GetRenderWindow ()
+        self.renwin.AddRenderer (self.ren)
         
         if stereo:
             self.renwin.SetStereoType(stereo[1])
             self.renwin.SetStereoRender(stereo[0])
             
-	self.camera = self.ren.GetActiveCamera()        
+        self.camera = self.ren.GetActiveCamera()        
         # this sets up the tkwidget and forces _CurrentRenderer
         # etc. to be updated
         self.tkwidget.UpdateRenderer (0.0, 0.0)
 
-	buttonFrame=Tkinter.Frame (self.frame, relief='sunken', bd=2)
-	buttonFrame.pack (side='bottom')
+        buttonFrame=Tkinter.Frame (self.frame, relief='sunken', bd=2)
+        buttonFrame.pack (side='bottom')
         Tkinter.Label (buttonFrame, text="View:").pack (side='left')
-	Tkinter.Button(buttonFrame, text="+X",
+        Tkinter.Button(buttonFrame, text="+X",
                        command=self.x_plus_view).pack(side='left')
-	Tkinter.Button(buttonFrame, text="-X",
+        Tkinter.Button(buttonFrame, text="-X",
                        command=self.x_minus_view).pack(side='left')
-	Tkinter.Button(buttonFrame, text="+Y",
+        Tkinter.Button(buttonFrame, text="+Y",
                        command=self.y_plus_view).pack(side='left')
-	Tkinter.Button(buttonFrame, text="-Y",
+        Tkinter.Button(buttonFrame, text="-Y",
                        command=self.y_minus_view).pack(side='left')
-	Tkinter.Button(buttonFrame, text="+Z",
+        Tkinter.Button(buttonFrame, text="+Z",
                        command=self.z_plus_view).pack(side='left')
-	Tkinter.Button(buttonFrame, text="-Z",
+        Tkinter.Button(buttonFrame, text="-Z",
                        command=self.z_minus_view).pack(side='left')
-	#Tkinter.Button(buttonFrame, text="Default View",
+        #Tkinter.Button(buttonFrame, text="Default View",
         #               command=self.reset_zoom).pack(side='left')
-	Tkinter.Button(buttonFrame, text="Isometric",
+        Tkinter.Button(buttonFrame, text="Isometric",
                        command=self.isometric_view).pack(side='left')
 
         self.set_background (Common.config.bg_color)
@@ -147,23 +148,33 @@ class RenderWindow (Base.Objects.VizObject):
     def __del__ (self):
         debug ("In RenderWindow::__del__ ()")
         pass
-    
+
+    def disable_render(self):        
+        """Use this to disable any rendering via Render() calls."""
+        debug ("In RenderWindow::disable_render ()")
+        self._do_render = 0
+
+    def enable_render(self):
+        """Use this to enable rendering via Render() calls."""
+        debug ("In RenderWindow::enable_render ()")
+        self._do_render = 1
+        
     def update_view (self, x, y, z, vx, vy, vz):
         debug ("In RenderWindow::update_view ()")
         self.camera.SetFocalPoint (0.0, 0.0, 0.0)
         self.camera.SetPosition (x, y, z)
         self.camera.SetViewUp (vx, vy, vz)
-	self.ren.ResetCamera()
-	self.renwin.Render()
+        self.ren.ResetCamera()
+        self.renwin.Render()
 
     def x_plus_view (self): 
         debug ("In RenderWindow::x_plus_view ()")
         self.update_view (self.def_pos, 0, 0, 0, 0, 1)
-	
+        
     def x_minus_view (self): 
         debug ("In RenderWindow::x_minus_view ()")
         self.update_view (-self.def_pos, 0, 0, 0, 0, 1)
-	
+        
     def z_plus_view (self): 
         debug ("In RenderWindow::z_plus_veiw ()")
         self.update_view (0, 0, self.def_pos, 0, 1, 0)
@@ -175,7 +186,7 @@ class RenderWindow (Base.Objects.VizObject):
     def y_plus_view (self): 
         debug ("In RenderWindow::y_plus_view ()")
         self.update_view (0, self.def_pos, 0, 1, 0, 0)
-	
+        
     def y_minus_view (self): 
         debug ("In RenderWindow::y_minus_view ()")
         self.update_view (0, -self.def_pos, 0, 1, 0, 0)
@@ -187,12 +198,12 @@ class RenderWindow (Base.Objects.VizObject):
 
     def reset_zoom (self): 
         debug ("In RenderWindow::reset_zoom ()")
-	self.ren.ResetCamera ()
-	self.renwin.Render ()
+        self.ren.ResetCamera ()
+        self.renwin.Render ()
 
     def add_actors (self, actors):
         """ Adds a single actor or a tuple or list of actors to the
-	renderer."""
+        renderer."""
         debug ("In RenderWindow::add_actors ()")
         if type (actors) is types.ListType or \
            type (actors) is types.TupleType:            
@@ -210,7 +221,7 @@ class RenderWindow (Base.Objects.VizObject):
         if type (actors) is types.ListType or \
            type (actors) is types.TupleType:
             for actor in actors:
-		self.remove_actors (actor)
+                self.remove_actors (actor)
         else:
             # warning: I dont check if the type is valid.
             if actors:
@@ -249,11 +260,11 @@ class RenderWindow (Base.Objects.VizObject):
                                   defaultextension=".ppm",
                                   filetypes=[("PPM images", "*.ppm"), 
                                              ("All files", "*")])
-	if len (file_name) != 0:
+        if len (file_name) != 0:
             Common.state.busy ()
-	    self.renwin.SetFileName (file_name)
-	    self.lift ()
-	    self.renwin.SaveImageAsPPM ()
+            self.renwin.SetFileName (file_name)
+            self.lift ()
+            self.renwin.SaveImageAsPPM ()
             Common.state.idle ()
 
     def save_bmp (self, file_name=""): 
@@ -577,8 +588,8 @@ class RenderWindow (Base.Objects.VizObject):
         del self.pipe_objs
         del self.root
         del self.renwin
-	self.tkwidget.destroy()
-	self.frame.destroy ()
+        self.tkwidget.destroy()
+        self.frame.destroy ()
 
     def no_quit (self, event=None):
         debug ("In RenderWindow::no_quit ()")
@@ -587,17 +598,18 @@ class RenderWindow (Base.Objects.VizObject):
         tkMessageBox.showerror ("ERROR", msg)
 
     def lift (self): 
-	"Lift the window to the top. Useful for scene dumps."
+        "Lift the window to the top. Useful for scene dumps."
         debug ("In RenderWindow::lift ()")
         top = self.frame.winfo_toplevel ()
-	top.deiconify ()
-	top.lift ()
-	self.frame.update_idletasks ()
-	self.Render ()
+        top.deiconify ()
+        top.lift ()
+        self.frame.update_idletasks ()
+        self.Render ()
 
     def Render (self): 
         debug ("In RenderWindow::Render ()")
-	self.renwin.Render ()
+        if self._do_render:
+            self.renwin.Render ()
 
     def GetActiveCamera (self):
         debug ("In RenderWindow::GetActiveCamera ()")
@@ -609,24 +621,24 @@ class RenderWindow (Base.Objects.VizObject):
 
     def get_renderer (self): 
         debug ("In RenderWindow::get_renderer ()")
-	return self.ren
+        return self.ren
 
     def get_render_window (self): 
         debug ("In RenderWindow::get_render_window ()")
-	return self.renwin
+        return self.renwin
 
     def get_vtk_render_window (self): 
         debug ("In RenderWindow::get_vtk_render_window ()")
-	return self.renwin
+        return self.renwin
 
     def get_light_manager (self): 
         debug ("In RenderWindow::get_light_manager ()")
-	return self.light_mgr
+        return self.light_mgr
 
     def set_background (self, clr=Common.config.bg_color): 
         debug ("In RenderWindow::set_background ()")
-	self.ren.SetBackground (*clr)
-	#self.Render ()
+        self.ren.SetBackground (*clr)
+        #self.Render ()
 
     def save_config (self, file): 
         "Save the current configuration to file."
