@@ -9,8 +9,8 @@ Copyright (c) 2003, Prabhu Ramachandran.
 """
 
 __author__ = "Prabhu Ramachandran <prabhu_r@users.sf.net>"
-__version__ = "$Revision: 1.5 $"
-__date__ = "$Date: 2003/10/16 08:53:44 $"
+__version__ = "$Revision: 1.10 $"
+__date__ = "$Date: 2005/01/05 19:07:01 $"
 
 import string, os
 import Base.Objects, Common
@@ -20,6 +20,7 @@ import vtkPipeline.vtkMethodParser
 import vtkPipeline.ConfigVtkObj
 
 debug = Common.debug
+vtk_version = float(vtkpython.vtkVersion().GetVTKVersion()[:3])
 
 def find_data_type (file_name):
     "Parses the named file to see what type of data there is."
@@ -111,8 +112,8 @@ class VtkXMLDataReader (Base.Objects.DataSource):
     def __init__ (self, renwin=None): 
         debug ("In VtkXMLDataReader::__init__ ()")
         Base.Objects.DataSource.__init__ (self)
-	self.scalar_lst = [] # avaliable scalars
-	self.vector_lst = []
+        self.scalar_lst = [] # avaliable scalars
+        self.vector_lst = []
         self.tensor_lst = []
         self.point_attr = None
         self.cell_attr = None
@@ -122,25 +123,31 @@ class VtkXMLDataReader (Base.Objects.DataSource):
         self.vector_var = Tkinter.StringVar ()
         self.tensor_var = Tkinter.StringVar ()
         self.data_types = {'StructuredGrid': 'STRUCTURED_GRID',
+                           'PStructuredGrid': 'STRUCTURED_GRID',
                            'StructuredPoints': 'STRUCTURED_POINTS',
                            'ImageData': 'IMAGE_DATA',
+                           'PImageData': 'IMAGE_DATA',
                            'RectilinearGrid': 'RECTILINEAR_GRID',
+                           'PRectilinearGrid': 'RECTILINEAR_GRID',
                            'PolyData': 'POLYDATA',
-                           'UnstructuredGrid': 'UNSTRUCTURED_GRID'}
+                           'PPolyData': 'POLYDATA',
+                           'UnstructuredGrid': 'UNSTRUCTURED_GRID',
+                           'PUnstructuredGrid': 'UNSTRUCTURED_GRID'}
 
     def __del__ (self): 
         debug ("In VtkXMLDataReader::__del__ ()")
     
     def initialize (self, file_name): 
-	"Overload this if reqd. Use the existing functions if possible."    
+        "Overload this if reqd. Use the existing functions if possible."    
         debug ("In VtkXMLDataReader::initialize ()")
+        Base.Objects.DataSource.initialize (self, file_name)
         Common.state.busy ()
-	self.file_name = file_name
-	self.create_reader ()
-	self.set_file_name (file_name)
+        self.file_name = file_name
+        self.create_reader ()
+        self.set_file_name (file_name)
         self.reader.Update ()
         self.setup_defaults ()
-	self.reader.GetOutput().Update ()
+        self.reader.GetOutput().Update ()
         self.update_references ()
         Common.state.idle ()
         
@@ -212,21 +219,21 @@ class VtkXMLDataReader (Base.Objects.DataSource):
         self.tensor_data_name = name
 
     def create_reader (self): 
-	"Create the corresponding reader."
+        "Create the corresponding reader."
         debug ("In VtkXMLDataReader::create_reader ()")
-	# set up the reader	
-	if self.file_name == "":
-	    raise IOError, "No filename specifed for the data handler!"
+        # set up the reader     
+        if self.file_name == "":
+            raise IOError, "No filename specifed for the data handler!"
 
         gt = find_data_type (self.file_name)
-	self.grid_type = self.data_types[gt]
+        self.grid_type = self.data_types[gt]
         self.reader = eval('vtkpython.vtkXML%sReader()'%gt)
 
     def set_scalar_name (self, scalar): 
         debug ("In VtkXMLDataReader::set_scalar_name ()")
         assert (scalar in self.scalar_lst), \
                "Sorry, no scalar called %s available"%scalar
-	self.scalar_data_name = scalar
+        self.scalar_data_name = scalar
         out = self.reader.GetOutput()
         pd = out.GetPointData()
         cd = out.GetCellData()
@@ -234,7 +241,7 @@ class VtkXMLDataReader (Base.Objects.DataSource):
             pd.SetActiveScalars(scalar)
         else:
             cd.SetActiveScalars(scalar)
-	self.Update ()
+        self.Update ()
         self.force_update()
         self.update_references ()
 
@@ -242,7 +249,7 @@ class VtkXMLDataReader (Base.Objects.DataSource):
         debug ("In VtkXMLDataReader::set_vector_name ()")
         assert (vector in self.vector_lst), \
                "Sorry, no vector called %s available"%vector
-	self.vector_data_name = vector
+        self.vector_data_name = vector
         out = self.reader.GetOutput()
         pd = out.GetPointData()
         cd = out.GetCellData()
@@ -250,7 +257,7 @@ class VtkXMLDataReader (Base.Objects.DataSource):
             pd.SetActiveVectors(vector)
         else:
             cd.SetActiveVectors(vector)
-	self.Update ()
+        self.Update ()
         self.force_update()
         self.update_references ()
 
@@ -258,7 +265,7 @@ class VtkXMLDataReader (Base.Objects.DataSource):
         debug ("In VtkXMLDataReader::set_tensor_name ()")
         assert (tensor in self.tensor_lst), \
                "Sorry, no tensor called %s available"%tensor
-	self.tensor_data_name = tensor
+        self.tensor_data_name = tensor
         out = self.reader.GetOutput()
         pd = out.GetPointData()
         cd = out.GetCellData()
@@ -266,7 +273,7 @@ class VtkXMLDataReader (Base.Objects.DataSource):
             pd.SetActiveTensors(tensor)
         else:
             cd.SetActiveTensors(tensor)
-	self.Update ()
+        self.Update ()
         self.force_update()
         self.update_references ()
 
@@ -277,36 +284,38 @@ class VtkXMLDataReader (Base.Objects.DataSource):
         debug ("In VtkXMLDataReader::set_file_name ()")
         self.file_name = file_name
         self.reader.SetFileName(file_name)
-	
+        
     def get_scalar_list (self): 
         debug ("In VtkXMLDataReader::get_scalar_list ()")
-	return self.scalar_lst
+        return self.scalar_lst
 
     def get_vector_list (self): 
         debug ("In VtkXMLDataReader::get_vector_list ()")
-	return self.vector_lst
+        return self.vector_lst
 
     def get_tensor_list (self): 
         debug ("In VtkXMLDataReader::get_tensor_list ()")
-	return self.tensor_lst
-	    
+        return self.tensor_lst
+            
     def GetOutput (self): 
-	"Get the reader's output. "
+        "Get the reader's output. "
         debug ("In VtkXMLDataReader::GetOutput ()")
-	return self.reader.GetOutput ()
+        return self.reader.GetOutput ()
 
     def get_output (self): 
         debug ("In VtkXMLDataReader::get_output ()")
-	return self.reader.GetOutput ()
+        return self.reader.GetOutput ()
 
     def force_update(self):
         """Used to force the pipeline to update. This is necessary
         since the consumers do not update when the output is
         modified."""
         debug ("In VtkXMLDataReader::force_update ()")
-        out = self.GetOutput()
-        for i in range(out.GetNumberOfConsumers()):
-            out.GetConsumer(i).Modified()
+        if vtk_version < 4.5:
+            # This does not seem necessary with VTK-4.5 and above.
+            out = self.GetOutput()
+            for i in range(out.GetNumberOfConsumers()):
+                out.GetConsumer(i).Modified()
 
     def Update (self): 
         debug ("In VtkXMLDataReader::Update ()")
@@ -317,7 +326,7 @@ class VtkXMLDataReader (Base.Objects.DataSource):
 
     def get_reader (self): 
         debug ("In VtkXMLDataReader::get_reader ()")
-	return self.reader
+        return self.reader
 
     def get_render_window (self): 
         debug ("In VtkXMLDataReader::get_render_window ()")
@@ -342,6 +351,8 @@ class VtkXMLDataReader (Base.Objects.DataSource):
             self.file_name = f_name
         else:
             self.file_name = get_file_name (f_name)
+        
+        Base.Objects.DataSource.initialize (self, self.file_name)
         sd = file.readline()[:-1]
         vd = file.readline()[:-1]
         td = file.readline()[:-1]
@@ -352,8 +363,8 @@ class VtkXMLDataReader (Base.Objects.DataSource):
         p.load (self.reader, file)
         # Correct the filename because the stored filename uses an
         # absolute path.
-	self.set_file_name(self.file_name)
-	self.reader.Update ()
+        self.set_file_name(self.file_name)
+        self.reader.Update ()
         
         pa, ca = get_attribute_lists(self.reader.GetOutput())
         self.point_attr, self.cell_attr = pa, ca
@@ -436,6 +447,9 @@ class VtkXMLDataReader (Base.Objects.DataSource):
         lab = Tkinter.Label (frame, text="File Name: "+
                              os.path.basename (self.file_name))
         lab.pack (side='top', fill='both', expand=1)
+        self.file_name_label = lab
+        
+        self.make_timestep_gui(frame)
         self.scalar_gui (frame)
         self.vector_gui (frame)
         self.tensor_gui (frame)
@@ -503,6 +517,8 @@ class VtkXMLDataReader (Base.Objects.DataSource):
         self.reader.Modified ()
         self.reader.Update ()
         pa, ca = get_attribute_lists(self.reader.GetOutput())
+        change_in_attributes = (self.point_attr != pa) or \
+                               (self.cell_attr != ca)
         self.point_attr, self.cell_attr = pa, ca
         self.scalar_lst = pa['scalars'] + ca['scalars']
         self.vector_lst = pa['vectors'] + ca['vectors']
@@ -540,9 +556,10 @@ class VtkXMLDataReader (Base.Objects.DataSource):
         self.renwin.Render()
         # the following changes the gui so that any changes in the
         # data are reflected in the gui.
-        geom = self.root.geometry()
-        master = self.root.master
-        self.close_gui ()
-        self.configure (master)
-        self.root.geometry(geom[string.find(geom, '+'):])
+        if self.root and self.root.winfo_exists() and change_in_attributes:
+            geom = self.root.geometry()
+            master = self.root.master
+            self.close_gui ()
+            self.configure (master)
+            self.root.geometry(geom[string.find(geom, '+'):])
         Common.state.idle ()
